@@ -3,6 +3,29 @@ import sqlite3
 from fish_tool.log_tool import logs, InfoError
 
 
+class ScrollIter:
+    # 包装一个迭代器， 使其增加长度属性， 便于tqdm等显示进度条
+    def __init__(self, db, table, condition=None, num=100):
+        self.return_num = 0
+        self.db = db
+        self.table = table
+        self.condition = condition
+        self.num = num
+        self.total = db.num(table, condition=condition)
+        self.data = []
+
+    def __len__(self):
+        return self.total
+
+    def __iter__(self):
+        while self.return_num < self.total:
+            if self.data:
+                for one in self.data:
+                    self.return_num += 1
+                    yield one
+            self.data = self.db.select(self.table, condition=self.condition, num=self.num, start=self.return_num)
+
+
 class SqliteTool:
     def __init__(self, path):
         self.db = sqlite3.connect(path)
@@ -76,13 +99,15 @@ class SqliteTool:
         out = line[0]
         return out
 
-    def select(self, table, condition=None, num=None):
+    def select(self, table, condition=None, num=None, start=None):
         cur = self.db.cursor()
         sql = f'select * from {table}'
         if condition:
             sql += f' where {condition}'
         if num is not None:
             sql += f' limit {num}'
+        if start is not None:
+            sql += f' OFFSET {start}'
         cur.execute(sql)
         fields = [t[0] for t in cur.description]
         out = []
@@ -95,6 +120,9 @@ class SqliteTool:
             out.append(obj)
         return out
 
+    def scroll(self, table, condition=None, num=100):
+        return ScrollIter(self, table, condition=condition, num=num)
+
     def commit(self):
         self.db.commit()
 
